@@ -24,7 +24,7 @@
   * @author Samuel Mimram
   *)
 
-(* $Id: mad.ml 4672 2007-10-17 16:48:18Z metamorph68 $ *)
+(* $Id: mad.ml 5568 2008-05-18 00:44:33Z metamorph68 $ *)
 
 type mad_file
 
@@ -122,9 +122,18 @@ let duration file =
         assert false
       with _ -> !bytes
   in
-    try
-      let mf = openfile file in
-      let samplefreq = samplefreq file in
-        (float (8 * bytes mf)) /.
-        (float (wav_output_channels*wav_output_sample_size*samplefreq))
-    with _ -> 0.
+  try
+    let mf = openfile file in
+    let ret = 
+      try
+        let samplefreq = samplefreq file in
+          (float (8 * bytes mf)) /.
+          (float (wav_output_channels*wav_output_sample_size*samplefreq))
+      with _ -> 0.
+    in
+    try 
+      close mf ;
+      ret
+    with _ -> ret
+  with
+    | _ -> 0.
