@@ -24,7 +24,7 @@
  * @author Samuel Mimram, David Baelde
  */
 
-/* $Id: mad_stubs.c 5355 2008-04-02 09:15:13Z metamorph68 $ */
+/* $Id: mad_stubs.c 5859 2008-09-02 08:46:03Z metamorph68 $ */
 
 #include <caml/alloc.h>
 #include <caml/callback.h>
@@ -39,6 +39,8 @@
 #include <errno.h>
 #include <string.h>
 #include <assert.h>
+#include <netinet/in.h>
+#include <stdint.h>
 #include <mad.h>
 
 #define BUFFER_SIZE 16*1024
@@ -136,15 +138,52 @@ static inline madfile_t* create_mf()
   return(mf);
 }
 
+/* Returns a "normal" 4-bytes integer from
+ * a synchsafe integer */
+int unsynchsafe(unsigned int in) {
+        int out=0, i;
+        unsigned int mask=0x7F000000;
+ 
+        for(i=0; i<4; ++i) {
+                out >>= 1;
+                out |= in & mask;
+                mask >>= 8;
+        }
+ 
+        return out;
+}
+
 CAMLprim value ocaml_mad_openfile(value file)
 {
   CAMLparam1(file);
   CAMLlocal1(block);
   madfile_t *mf;
+  FILE *fd = fopen(String_val(file), "r");
+
+  /* Remove ID3 tag 
+   * Ref: http://www.id3.org/id3v2.4.0-structure */
+  char id3_header[3];
+  uint32_t id3_len;
+  int footer_len = 0;
+  fread(&id3_header,sizeof(char),3,fd);
+  /* Check for ID3 tag magic */
+  if ((id3_header[0] == 0x49) & 
+      (id3_header[1] == 0x44) & 
+      (id3_header[2] == 0x33))
+  { /* Read version and flags */
+    fread(&id3_header,sizeof(char),3,fd);
+    /* Check for footer flag */
+    if (id3_header[2] & 0x10) // 0b00010000 doesn't seem to work will all compilers..
+      footer_len = 10;
+    /* Get synchsafe len of ID3 tag */
+    fread(&id3_len,sizeof(char),4,fd);
+    fseek(fd,unsynchsafe(ntohl(id3_len))+footer_len,SEEK_CUR);
+  } else 
+    rewind(fd);
 
   mf = create_mf();
   mf->read_func = (value)NULL;
-  mf->fd = fopen(String_val(file), "r");
+  mf->fd = fd;
   if (!mf->fd)
     caml_raise_with_arg(*caml_named_value("mad_exn_openfile_error"), caml_copy_string(strerror(errno)));
   mf->buf = (unsigned char*)malloc(BUFFER_SIZE);
